const { merge } = require('webpack-merge');
const TerserPlugin = require('terser-webpack-plugin');
const CssMinimizerPlugin = require('css-minimizer-webpack-plugin');
const { getProjectPath, deleteFolders } = require('../common/utils');
const baseConfig = require('./webpack.base');

module.exports = function (appConfig, extendEnv) {
  const outputPath = getProjectPath(appConfig.buildOutput.path);
  deleteFolders(outputPath);

  return merge(baseConfig(true, appConfig, extendEnv), {
    // 打包的入口文件
    entry: getProjectPath(appConfig.buildEntry),
    // 打包的出口
    output: {
      publicPath: appConfig.buildOutput.publicPath || '/',
      path: outputPath,
      // 入口文件生产的js
      filename: `js/[name].chunk.js?v=${extendEnv.version}`,
      // 非入口文件生产的js
      chunkFilename: `js/[name].chunk.js?v=${extendEnv.version}`
    },
    optimization: {
      concatenateModules: true,
      minimizer: [
        // 压缩js
        new TerserPlugin({
          terserOptions: {
            parse: {
              ecma: 8
            },
            compress: {
              ecma: 5,
              warnings: false,
              comparisons: false,
              inline: 2
            },
            mangle: {
              safari10: true
            },
            output: {
              ecma: 5,
              comments: false,
              ascii_only: true
            }
          }
        }),
        // 压缩css
        new CssMinimizerPlugin()
      ],
      splitChunks: {
        chunks: 'initial',
        minSize: 0,
        // 按需加载时并行请求的最大数量
        maxAsyncRequests: 5,
        // 入口位置最大的请求数量
        maxInitialRequests: 3,
        automaticNameDelimiter: '~',
        cacheGroups: {
          commons: {
            test: /[\\/]node_modules[\\/]/,
            chunks: 'initial',
            name: 'vendors',
            minChunks: 1,
            priority: 10
          }
        }
      },
      runtimeChunk: {
        name: 'manifest'
      }
    }
  });
};
